#include "decl.h"

double calc_dt_relativistic(double t,struct ephemeris_record eph,int sat)

     // This function calculates a relativistic term for the GPS clocks.
     // See GPS SPS Signal Specification, AD 04-03
     //
     // Input  t                    : nominal time
     //        eph                  : last GPS ephemerides
     //        sat                  : GPS satellite
     //
     // Output calc_dt_relativistic : correction term

{
  double mu=3.986005e14;
  double A,n_0,n,t_oe_f,t_k,M_k,E_k,dt_r;
  double F=-4.442807633e-10;
  A=pow(eph.sqtA[sat-1],2.0);
  n_0=pow(mu/pow(A,3.0),0.5);
  n=n_0+eph.dn[sat-1];

  t_oe_f=eph.t_oe[sat-1]+7.0*86400.0*eph.week_n[sat-1];

  t_k=t-t_oe_f;

  M_k=eph.M0[sat-1]+n*t_k;

  /* translate mean anomaly into eccentric anomaly */
  E_k=ecce_anom(M_k,eph.e[sat-1],3);

  dt_r=F*eph.e[sat-1]*eph.sqtA[sat-1]*sin(E_k);

  return dt_r;

}



