#include "decl.h"

int crc_ok(unsigned char raw_message_bytes[max_bytes],int message_length)

     // This function calculates the CRC checksum for an EGNOS message
     // and compares it with the transmitted CRC in the message header.
     //
     // The code used for the CRC32 check is taken from the document
     // "IETF RFC.1662 CRC32"
     // which is listed as an "applicable document" for the
     // EGNOS CCF Interface Control Document (CCF ICD)
     //
     // Input  raw_message_bytes : message to be checked
     //        message_length    : number of bytes in the message
     //
     // Output crc_ok            : (boolean) checksum agreement (1/0)

{
  int i;
  u32 test_crc;
  unsigned char header_crc[4];

  // extract CRC from the message header bytes
  for (i=0; i < 4; i++) header_crc[i]=raw_message_bytes[i+27];
  // reset CRC bytes before checking the message
  for (i=0; i < 4; i++) raw_message_bytes[i+27]=0;

  test_crc=pppfcs32(PPPINITFCS32,raw_message_bytes,message_length);

  // check if resulting CRC is 2-complements of header CRC
  if ((header_crc[0]+((test_crc>>24)&255)==255)&&
      (header_crc[1]+((test_crc>>16)&255)==255)&&
      (header_crc[2]+((test_crc>> 8)&255)==255)&&
      (header_crc[3]+((test_crc>> 0)&255)==255))
    return 1;
  else
    return 0;
}
