#define lin_end "\r\n"
// #define lin_end "\n"

#define control_file_name "station_data.ctl"

#define NSAT 32
#define header_length 31
#define max_bytes 2000
#define max_n_rims 100
#define PI            3.14159265358979323846

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include "rfc1662_def.h"


struct GNSS_record{int status[NSAT]; double L1[NSAT]; int C_N0_L1[NSAT]; 
           double L2[NSAT]; int C_N0_L2[NSAT]; double P1[NSAT], P2[NSAT];
                  };
struct measurement_record{double GPS_secs; struct GNSS_record GPS;
           struct GNSS_record GLONASS; struct GNSS_record GEO;
           int writeable;
                         };
struct ephemeris_record{double iode[NSAT],C_rs[NSAT],dn[NSAT],M0[NSAT],C_uc[NSAT],e[NSAT],C_us[NSAT],
                         sqtA[NSAT],t_oe[NSAT],C_ic[NSAT],Omega_0[NSAT],C_is[NSAT],i_0[NSAT],
                         C_rc[NSAT],omega[NSAT],Omega_dot[NSAT],i_dot[NSAT],week_n[NSAT],
                         a_f0[NSAT],a_f1[NSAT],t_oc[NSAT],t_gd[NSAT];
                         int eph_status[NSAT],raw_frames[NSAT][210],frame_status[NSAT];
                 };
int read_command_line(int *time_interval_rinex,int *clock_tags,int *gps_only,int *day_break,
                   char rims_names[max_n_rims][80],char rinex_name[80],char run_by_name[20],
		      char rims_sta[5],int argc, char *argv[]);
void name_sort(char final_list[max_n_rims][80],char temp_list[max_n_rims][80],
               int n_names,int sorting);
int extract_rims_station(char rims_sta[5],char rimsf_name[80]);
void print_usage(void);
int extract_station_data(char rin_head[18][81],char rinex_sta[4],double c_n0_lim[6],double appr_pos[3],
			 int samp_interv,int gps_only,char run_by_name[20],char rims_sta[5]);
int read_ctl_str(char out_str[60],char type_str[20],int n_read_char,int line_nr,FILE *fd_ctl);
int read_ctl_dbl(double *out_dbl,char type_str[20],int line_nr,FILE *fd_ctl);
struct measurement_record reset_data(void);
struct ephemeris_record init_ephemeris(FILE *fd_rims);
void  read_raw_message(unsigned char raw_message_bytes[max_bytes],int *read_status,FILE *fd_rims);
int crc_ok(unsigned char raw_message_bytes[max_bytes],int message_length);
struct measurement_record extract_measurements(struct measurement_record old_measurements,
					   unsigned char in_bytes[max_bytes],int gps_only);
struct GNSS_record decode_GNSS(struct GNSS_record old_record,int system_type,
    			          int n_sats_obs,unsigned char in_bytes[max_bytes]);
double decode_phase(unsigned char in_bytes[4],int old_exists,double old_phase);
double decode_pseudo_range(unsigned char in_bytes[5]);
double decode_diff_pseudo_range(unsigned char in_bytes[2]);
void extract_ephemerides(struct ephemeris_record *ephemerides,
                double old_secs,unsigned char in_bytes[max_bytes]);
void get_nav_bytes(int nav_bytes[NSAT][7],int bytes_read[NSAT],int n_sats_obs,
                         unsigned char in_bytes[max_bytes]);
void update_eph(struct ephemeris_record *eph,int nav_bytes[NSAT][7],
                      int bytes_read[NSAT],double secs,double old_secs);
void decode_frame(unsigned long frame[50],int raw_frame[210]);
void decode_eph(struct ephemeris_record *eph,int *decoding_ok,
                                  unsigned long frame[50],int i);
int calc_dt(double *dt_receiver,double GPS_secs,struct ephemeris_record GPS_eph,double 
                                     approx_position[3],struct GNSS_record GPS_record);
void get_sat_coord(double coord[3],double t,struct ephemeris_record eph,int sat);
double ecce_anom(double M,double e,int n_iter);
double calc_dt_relativistic(double t,struct ephemeris_record eph,int sat);
int day_change(int *day_number, double GPS_secs);
void mk_rinexf_name(char rinexf_name[80],char rinex_station[4],int day_number,int day_break,int rinexf_open);
void write_header(char rinex_header[18][81],double GPS_secs,FILE *fd_rinex);
void s2ymdhms(int dat[6], long s);
long ymdhms2s(int dat[6],int leap_secs);
int c_n0_map(int C_N0,double c_n0_lim[3]);
void write_rinex_record(struct measurement_record measurement_data,struct ephemeris_record GPS_eph,
         double approx_position[3],int clock_tags,double c_n0_lim[6],int gps_only,FILE *fd);
void write_GNSS_system(struct GNSS_record GNSS_data,double c_n0_lim[6],int system_type,FILE *fd);
u32 pppfcs32(register u32 fcs, register unsigned char *cp, register int len);
