#include "decl.h"

void decode_eph(struct ephemeris_record *eph,int *decoding_ok,
                                  unsigned long frame[50],int i)

     // This function decode a frame of GPS ephemerides.
     // For the frame structure see GPS SPS Signal Specification, AD 04-03
     //
     // Input  eph         : present GPS ephemerides
     //        frame       : frame to decode
     //        i           : ephemeris item # (satellite number - 1)
     //
     // Output eph         : updated GPS ephemerides
     //        decoding_ok : (boolean) 1 if decoding is ok

{
  int sgn_bit;
  double iode,sqtA,i_0,mantissa;

  // Issue of data
  iode=(double)(frame[12] >> 22);

  // Extract sqrt(A) and inclination i_0 to test if data seem OK
   mantissa=(double)(((frame[17]&16320) << 18)+(frame[18] >> 6)); sgn_bit=0;
  sqtA=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-19.0);
   mantissa=(double)(((frame[24]&16320) << 18)+(frame[25] >> 6)); sgn_bit=((frame[24]&16383) >> 13);
  i_0=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-31.0)*PI;


  // Test if new Issue of data, and if sqtA and i_0 are reasonable before decoding
  if ((iode!=eph->iode[i])&&(sqtA>5150.0)&&(sqtA<5160.0)&&(i_0>0.85)&&(i_0<1.05))
    {
    // indicate that data are decoded
    *decoding_ok=1;
    eph->iode[i]=iode;
     mantissa=(double)((frame[12]&4194303) >> 6); sgn_bit=((frame[12]&4194303) >> 21);
    eph->C_rs[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0, -5.0);
     mantissa=(double)(frame[13] >> 14); sgn_bit=((frame[13]) >> 29);
    eph->dn[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-43.0)*PI;
     mantissa=(double)(((frame[13]&16320) << 18)+(frame[14] >> 6)); sgn_bit=((frame[13]&16383) >> 13);
    eph->M0[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-31.0)*PI;
     mantissa=(double)(frame[15] >> 14); sgn_bit=((frame[15]) >> 29);
    eph->C_uc[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-29.0);
     mantissa=(double)(((frame[15]&16320) << 18)+(frame[16] >> 6)); sgn_bit=0;
    eph->e[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-33.0);
     mantissa=(double)(frame[17] >> 14); sgn_bit=((frame[17]) >> 29);
    eph->C_us[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-29.0);
     mantissa=(double)(((frame[17]&16320) << 18)+(frame[18] >> 6)); sgn_bit=0;
    eph->sqtA[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-19.0);
     mantissa=(double)(frame[19] >> 14); sgn_bit=0;
    eph->t_oe[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,4.0);
     mantissa=(double)(frame[22] >> 14); sgn_bit=((frame[22]) >> 29);
    eph->C_ic[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-29.0);
     mantissa=(double)(((frame[22]&16320) << 18)+(frame[23] >> 6)); sgn_bit=((frame[22]&16383) >> 13);
    eph->Omega_0[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-31.0)*PI;
     mantissa=(double)(frame[24] >> 14); sgn_bit=((frame[24]) >> 29);
    eph->C_is[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-29.0);
     mantissa=(double)(((frame[24]&16320) << 18)+(frame[25] >> 6)); sgn_bit=((frame[24]&16383) >> 13);
    eph->i_0[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-31.0)*PI;
     mantissa=(double)(frame[26] >> 14); sgn_bit=((frame[26]) >> 29);
    eph->C_rc[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-5.0);
     mantissa=(double)(((frame[26]&16320) << 18)+(frame[27] >> 6)); sgn_bit=((frame[26]&16383) >> 13);
    eph->omega[i]=(mantissa - sgn_bit*pow(2.0,32.0))*pow(2.0,-31.0)*PI;
     mantissa=(double)(frame[28] >> 6); sgn_bit=((frame[28]) >> 29);
    eph->Omega_dot[i]=(mantissa - sgn_bit*pow(2.0,24.0))*pow(2.0,-43.0)*PI;
     mantissa=(double)((frame[29]&4194303) >> 8); sgn_bit=((frame[29]&4194303) >> 21);
    eph->i_dot[i]=(mantissa - sgn_bit*pow(2.0,14.0))*pow(2.0,-43.0)*PI;
    eph->week_n[i]=(double)(frame[2] >> 20)+1024.0;
     mantissa=(double)(frame[9] >> 8); sgn_bit=((frame[9]) >> 29);
    eph->a_f0[i]=(mantissa - sgn_bit*pow(2.0,22.0))*pow(2.0,-31.0);
     mantissa=(double)((frame[8]&4194303) >> 6); sgn_bit=((frame[8]&4194303) >> 21);
    eph->a_f1[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,-43.0);
     mantissa=(double)((frame[7]&4194303) >> 6); sgn_bit=0;
    eph->t_oc[i]=(mantissa - sgn_bit*pow(2.0,16.0))*pow(2.0,4.0);
     mantissa=(double)((frame[6]&16383) >> 6); sgn_bit=((frame[6]&16383) >> 13);
    eph->t_gd[i]=(mantissa - sgn_bit*pow(2.0,8.0))*pow(2.0,-31.0);
    }
  // indicate no decoding
  else *decoding_ok=0;
}


