#include "decl.h"

void decode_frame(unsigned long frame[50],int raw_frame[210])

     // This function takes a set of RIMS bytes, containing GPS 
     // ephemeris for one satellite, and put the back into the original
     // GPS ephemeris frame structure.
     // See EGNOS CCF Interface Control Document (CCF ICD) for raw_frame.
     // For the frame structure see GPS SPS Signal Specification, AD 04-03
     //
     // Input  raw_frame : RIMS ephemeris bytes to convert
     //
     // Output frame     : GPS ephemeris frame for one satellite

{
  unsigned long prel_frame;
  int i,j,last_D30,word_10[30];

  // Reconstruct the value of last bit in last subframe
  last_D30=1-(raw_frame[0] >> 7); 

  // loop over 5 subframes (each with 10 30-byte words)
  // that constitute a frame (i.e. 5*10 30-byte words)
  for (i=0; i < 5; i++)
    {
    // Raw RIMS navigation data in raw_frame is contained in groups of 7 bytes
    // where 50 bits (out of 7*8=56 bits) are valid.
    // Take 6 groups, 6*7=42 raw_frame bytes, (6*50 valid bits) and put them
    // into 30 10-byte words
    for (j=0; j < 6; j++)
      {
      word_10[5*j+0]=((raw_frame[42*i+7*j+0]   ) << 2)+ (raw_frame[42*i+7*j+1] >> 6);
      word_10[5*j+1]=((raw_frame[42*i+7*j+1]&63) << 4)+ (raw_frame[42*i+7*j+2] >> 4);
      word_10[5*j+2]=((raw_frame[42*i+7*j+2]&15) << 6)+ (raw_frame[42*i+7*j+3] >> 2);
      word_10[5*j+3]=((raw_frame[42*i+7*j+3]& 3) << 8)+ (raw_frame[42*i+7*j+4]     );
      word_10[5*j+4]=((raw_frame[42*i+7*j+5]   ) << 2)+ (raw_frame[42*i+7*j+6] >> 6);
      }

    // Reorganize 3 10-byte words into one 30-byte word. 
    // 10 such 30-byte word consitiutes a subframe 
    for (j=0; j < 10; j++)
      {
      prel_frame=word_10[3*j+0]*1048576 + word_10[3*j+1]*1024 + word_10[3*j+2];
      // If last bit in previous subframe=1 then use 2-complement
      if (last_D30)
        frame[10*i+j]=1073741824-prel_frame-1;
      else 
        frame[10*i+j]=prel_frame;
      last_D30=(prel_frame&1);
      }
    }
}
