#include "decl.h"

double decode_phase(unsigned char in_bytes[4],int old_exists,double old_phase)

     // This function decodes the RIMS carrier phase observables.
     // It compensates for possible data roll-over that occur since
     // the limited number of bytes don't cover typical phase changes.
     // See EGNOS CCF Interface Control Document (CCF ICD)
     // Annex 2, Receiver Accumulated Doppler
     //
     // Input  in_bytes     : RIMS raw measurement bytes to convert
     //        old_exists   : (boolean) old_phase valid
     //        old_phase    : last observation for this satellite
     //
     // Output decode_phase : new phase observation

{
  int n_add;
  double phase;

  phase = (double)(32768.0*in_bytes[0] +       128.0*in_bytes[1]+
                       0.5*in_bytes[2] + 0.001953125*in_bytes[3]);
  if (phase >= 4194304.0) phase = phase - 8388608.0;

  // If old data available compensate for possible data roll-over
  // by adding an integer times 2^23
  if (old_exists) {
    if (old_phase >= phase)
      n_add=(int)((old_phase - phase)/8388608.0 + 0.5);
    else
      n_add=(int)((old_phase - phase)/8388608.0 - 0.5);
    phase = phase + n_add*8388608.0; }

  return phase;
}

