#include "decl.h"

double ecce_anom(double M,double e,int n_iter)

     // This function calculates the eccentric anomaly of a
     // satellite orbit from its mean anomaly using 
     // Keplers's equation E=M+e*sin(E).
     // See GPS SPS Signal Specification, AD 04-03
     //
     // Input  M         : Mean anomaly (radians)
     //        e         : orbit eccentricity
     //        n_iter    : number of iterations to use
     //
     // Output ecce_anom : eccentric anomaly

{
  int i;  
  double E;

  // start value for E 
  E=M+e*sin(M);
  // Iterate to get improved E value
  for (i=0; i < n_iter; i++)
    E=M+e*sin(E);
  
  return E;
}
