#include "decl.h"

void extract_ephemerides(struct ephemeris_record *ephemerides,
                double old_secs,unsigned char in_bytes[max_bytes])

     // This function read the GPS 50 bits messages contained in the 
     // RIMS raw measurement message, and extracts GPS ephemerides 
     // to an updated ephemeris record.
     // See EGNOS CCF Interface Control Document (CCF ICD) for 
     // RIMS raw measurement message.
     // See EGNOS CCF Interface Control Document (CCF ICD) Annex 2
     // for GPS 50 bits messages.
     // See GPS SPS Signal Specification, AD 04-03, for ephemeris 
     // definitions.
     //
     // Input  ephemerides : present GPS ephemerides
     //        old_secs    : time in GPS seconds of last observation
     //        in_bytes    : RIMS raw measurement message bytes
     //
     // Output ephemerides : updated GPS ephemerides

{
  double secs;
  int i,section_type,n_sats;
  int message_length,section_start=header_length,section_length;
  int nav_bytes[NSAT][7],bytes_read[NSAT];

  // Extract the present GPS time from the EGNOS Std Data header
  secs=1024.0*7.0*86400.0;
  for (i=0; i < 6; i++) secs=secs+(double)in_bytes[i+13]*pow(256.0,(double)(3-i));
  secs=(double)((long)(secs));

  // read length of RIMS raw measurement message
  // (we know from before that its length is at least header_length+3)
  message_length=256*(int)in_bytes[3]+(int)in_bytes[4];

  section_type=(int)in_bytes[section_start];
  section_length=256*(int)in_bytes[section_start+1]+
                     (int)in_bytes[section_start+2];

  // check that section_length don't point beyond the message length
  // and that section_type is GPS(11)
  if ((section_start+section_length<=message_length)&&(section_type==11))
    {
    // derive number of satellites in the data from the section length
    n_sats=(section_length-3)/32;
    // check if number of satellites within boundaries,
    // and that the section length fits perfectly with the 
    // derived number of satellites
    if ((n_sats>0)&&(n_sats<=NSAT)&&(n_sats*32==section_length-3))
      {
      // read GPS 50 bits messages for navigation data
      get_nav_bytes(nav_bytes,bytes_read,n_sats,in_bytes+section_start+3);
      }
    }
      
  // update ephemerides with the content in the GPS 50 bits messages
  update_eph(ephemerides,nav_bytes,bytes_read,secs,old_secs);
}

