#include "decl.h"

void get_nav_bytes(int nav_bytes[NSAT][7],int bytes_read[NSAT],int n_sats_obs,
                         unsigned char in_bytes[max_bytes])

     // This function read the GPS 50 bits messages contained in the
     // RIMS raw measurement message
     // See EGNOS CCF Interface Control Document (CCF ICD) Annex 2
     // for GPS 50 bits messages.
     //
     // Input  n_sats_obs : number of satellites in the RIMS message
     //        in_bytes   : RIMS raw measurement message bytes
     //
     // Output nav_byte   : array of nav bytes for all GPS satellites
     //        bytes_read : (booleans) 1 if bytes are read for this satellite
 
{
  int i,j,sat;
  int n_bytes_per_sat=32;
  int decoding_error=0;


  // reset the boolean read indicators
  for (i=0; i < NSAT; i++) bytes_read[i]=0;

  // loop over all satellites in the RIMS message
  for (i=0; i < n_sats_obs; i++)
    // check if decoding ok so far
    if (!decoding_error)
    {
    sat=(int)in_bytes[n_bytes_per_sat*i];
    // check if satellite number is valid
    if ((sat<1)||(sat>NSAT))
      decoding_error=1;
    else
      {
      // put the new nav bytes in the output array
      for (j=0; j < 7; j++) nav_bytes[sat-1][j]=(int)in_bytes[j+25+n_bytes_per_sat*i];
      // mark that array element is valid
      bytes_read[sat-1]=1;
      }
    }
}

