#include "decl.h"

struct ephemeris_record init_ephemeris(FILE *fd_rims)

     // This function resets the status values of the 
     // ephemeris record, then read RIMS raw measurement 
     // messages until 5 sets of ephemeris are found. 
     // See EGNOS CCF Interface Control Document (CCF ICD) for
     // RIMS raw measurement message.
     // See GPS SPS Signal Specification, AD 04-03, for ephemeris
     // definitions.
     //
     // Input  fd_rims        : RIMS file
     //
     // Output init_ephemeris : initialized ephemerides

{
  struct ephemeris_record ephemerides;
  int i,read_status=-1,ephemeris_initialised=0,n_eph;
  unsigned char in_bytes[max_bytes];
  double old_secs=0;

  // reset status values
  for (i=0; i < NSAT; i++) ephemerides.frame_status[i]=0;
  for (i=0; i < NSAT; i++) ephemerides.eph_status[i]=0;

  // while RIMS file not ready and 5 ephemerides not yet found 
  while ((read_status<1)&&(!ephemeris_initialised))
    {
    read_raw_message(in_bytes,&read_status,fd_rims);
    // if in_bytes contains a valid message
    if (read_status<1) 
      {
      // extract ephemeris data from in_ bytes
      extract_ephemerides(&ephemerides,old_secs,in_bytes);

      // update the "old" (previous) time tag, needed for the
      // next turn, using the EGNOS Std Data header
      old_secs=1024.0*7.0*86400.0;
      for (i=0; i < 6; i++) old_secs+=(double)in_bytes[i+13]*pow(256.0,(double)(3-i));
      old_secs=(double)((long)(old_secs));

      // now, check if 5 ephemerides are found
      n_eph=0;
      for (i=0; i < NSAT; i++) if (ephemerides.eph_status[i]) n_eph++;
      if (n_eph>=5) ephemeris_initialised=1;
      }
    }
 
  return ephemerides;
}

