#include "decl.h"

void mk_rinexf_name(char rinexf_name[80],char rinex_station[4],int day_number,int day_break,int rinexf_open)

     // This function makes a name string for a new rinex file.
     //
     // Input  rinexf_name   : present name string
     //        rinex_station : station name string
     //        day_number    : present day number since GPS time start
     //        day_break     : (boolean) 1 if new file wanted at day boundary
     //        rinexf_open   : (boolean) 1 if a rinex file has already been opened
     //
     // Output rinexf_name   : updated name string

{
  int i,dat[6],new_year_dat[6],day_of_year;
  char new_file_name[20];


  // If a rinex file not yet open or it's not a day boundary
  // then don't update the present name if it's not empty.
  // Otherwise derive a standard rinex name as follows:
  if ((strlen(rinexf_name)==0)||((day_break)&&(rinexf_open)))
    {
    // Start with the station name
    strncpy(new_file_name,rinex_station,4);

    // derive the day_of_year=day number of the present year
    // from present day number - day number at last new years day 
    s2ymdhms(dat, (long)(day_number*86400));
    new_year_dat[0]=dat[0];
    new_year_dat[1]=1;
    new_year_dat[2]=1;
    new_year_dat[3]=0;
    new_year_dat[4]=0;
    new_year_dat[5]=0;
    day_of_year=day_number-ymdhms2s(new_year_dat,0)/86400+1;
    if (day_of_year < 10) sprintf(new_file_name+4,"00%1d",day_of_year);
    else if (day_of_year < 100) sprintf(new_file_name+4,"0%2d",day_of_year);
    else sprintf(new_file_name+4,"%3d",day_of_year);
    strncpy(new_file_name+7,"0.",2);
    if ((dat[0]%100) < 10) sprintf(new_file_name+9,"0%1d",(dat[0]%100));
    else sprintf(new_file_name+9,"%2d",(dat[0]%100));
    strcpy(new_file_name+11,"o");

    strcpy(rinexf_name,new_file_name);
    }

}
