#include "decl.h"

void read_raw_message(unsigned char raw_message_bytes[max_bytes],int *read_status,FILE *fd_rims)

     // This function reads the RIMS file until an ok 
     // RIMS raw measurement message is found.
     // If none is found read_status=1 is returned.
     // See EGNOS CCF Interface Control Document (CCF ICD) for
     // RIMS raw measurement message.
     // The description of the file structure is found in 
     // Annex A to CCF ICD: 9.1 CCF Archive Data to PACF
     //
     // Input  fd_rime           : RIMS file
     //
     // Output raw_message_bytes : RIMS raw measurement message bytes
     //        read_status       : 0 if reading ok, 1 if problems

{
  int i,message_length,epoch_ready=0;
  unsigned char dump_bytes[30];


  // If new file read (and dump) the 13 bytes representing the file header
  if (*read_status==-1) fread(dump_bytes,1,13,fd_rims);

  *read_status=0;

  while ((!epoch_ready)&&(*read_status<1))
    {
    // Read (and dump) the 12 bytes representing file timestamp and
    // a) data length if within a data field (use 
    // b) end-of-file indication if at end of file
    // (Instead of these data we use
    // the EGNOS header to determine message length, 
    // and find that we are at end of file the hard way) 
    fread(dump_bytes,1,12,fd_rims);

     /* read header */
    if ((fread(raw_message_bytes,1,header_length,fd_rims)) != header_length) *read_status=1;
    else
      {
      message_length=256*(int)raw_message_bytes[3]+(int)raw_message_bytes[4];

      /* Don't exceed raw_message_bytes allocation */
      /* and skip message if no bytes beyond first section length */
      if ((message_length > max_bytes)||(message_length <= header_length+3)) *read_status=1; 
      else
        {
        /* read remainder of the record */ 
        if ((fread(raw_message_bytes+header_length,1,message_length-header_length,fd_rims)) 
                                                     != message_length-header_length) *read_status=1;
        else
          {
	  // check if crc ok
	  // and message_identifier says raw GNSS measurement
          if ((crc_ok(raw_message_bytes,message_length))&&(raw_message_bytes[1]==10)) 
              epoch_ready=1;
          }
        }
      }
    }

}
