#include "decl.h"

void update_eph(struct ephemeris_record *eph,int nav_bytes[NSAT][7],
                      int bytes_read[NSAT],double secs,double old_secs)

     // This function updates the raw_frame with the content in 
     // the GPS 50 bits messages. If time is at the end of a 
     // 30 second raw_frame cycle then ephemeris data is decoded 
     // from the frame.
     // See EGNOS CCF Interface Control Document (CCF ICD) for raw_frame.
     // For the frame structure see GPS SPS Signal Specification, AD 04-03
     //
     // Input  nav_bytes  : array of RIMS GPS 50 bits messages
     //        bytes_read : (boolean) 1 if nav_bytes valid for this sat.
     //        secs       : present time in GPS seconds
     //        old_secs   : last time in GPS seconds
     //
     // Output eph        : updated GPS ephemerides
 
{
  int i,j,frame_offset,time_gap=1,decoding_status;
  unsigned long full_frame[50];


  // find position in the cyclic raw_frame for the present time
  frame_offset=(int)((long)(secs+0.5-2)%30);
  // check if there were any time gap since last data
  if (fabs(secs-old_secs-1.0)<0.5) time_gap=0;

  // loop over all possible satellites
  for (i=0; i < NSAT; i++)
    {
    // put data at its position in the raw_frame
    for (j=0; j < 7; j++) 
      eph->raw_frames[i][j+7*frame_offset]=nav_bytes[i][j];

    // mark if it is a successful start of a new 
    // raw frame for this satellite
    if ((frame_offset==0)&&(bytes_read[i]))
      eph->frame_status[i]=1;
    // mark if a frame cycle is broken for this satellite
    else if ((bytes_read[i]==0)||(time_gap))
      eph->frame_status[i]=0;

    // if at the end of the 30 second frame cycle
    // and the frame input never broken during these 30 s
    // then decode the frame and extract ephemeris
    // for this satellite
    if ((frame_offset==29)&&(eph->frame_status[i]))
      {
      // reconstruct GPS frame structure
      decode_frame(full_frame,eph->raw_frames[i]);
      // decode the bits into a new ephemeris 
      decode_eph(eph,&decoding_status,full_frame,i);
      // if decoding ok, mark that ephemeris exists for this satellite
      if (decoding_status) eph->eph_status[i]=1;
      }
    }
}


