#include "decl.h"

void write_GNSS_system(struct GNSS_record GNSS_data,double c_n0_lim[6],int system_type,FILE *fd)

     // This function writes the GNSS data from one system (GPS, GLONASS, or GEO)
     // for one point in time to the rinex file.
     // For the phase data a one digit snr value is calculated and written.
     // from the frame. See
     // "RINEX: The Receiver Independent Exchange Format Version 2.10"
     //    by Werner Gurtner, AIUB
     //
     // Input  GNSS_data   : present measurement data for this system type
     //        c_n0_lim    : rinex snr value mapping limits
     //        system_type : 0= GPS, 1= GLONASS, 2= GEO
     //        fd          : rinex file

{
  int i;

  // loop over all possible satellites
  for (i=0; i < NSAT; i++) {
    // bit 0 (lsb) of "status" set if L1 data exists
    if (GNSS_data.status[i]&1)  
      // c_n0_map converts C_N0_L1 to a one digit value
      fprintf(fd,"%14.3f4%1d",GNSS_data.L1[i],c_n0_map(GNSS_data.C_N0_L1[i],c_n0_lim));
    // if L1 doesn't exist but there are other data for this satellite: use blanks
    else if (GNSS_data.status[i]) fprintf(fd,"                ");
    // bit 1 of "status" set if L2 data exists (only for GPS)
    if ((GNSS_data.status[i]&2)&&(system_type==0))  
      fprintf(fd,"%14.3f4%1d",GNSS_data.L2[i],c_n0_map(GNSS_data.C_N0_L2[i],c_n0_lim+3));
    // if L2 doesn't exist but there are other data for this satellite: use blanks
    else if (GNSS_data.status[i]) fprintf(fd,"                ");
    // bit 2 of "status" set if P1(C1) data exists
    if (GNSS_data.status[i]&4) fprintf(fd,"%14.3f  ",GNSS_data.P1[i]);
    // if P1(C1) doesn't exist but there are other data for this satellite: use blanks
    else if (GNSS_data.status[i]) fprintf(fd,"                ");
    // bit 3 of "status" set if P2 data exists (only for GPS)
    if ((GNSS_data.status[i]&8)&&(system_type==0)) fprintf(fd,"%14.3f  ",GNSS_data.P2[i]);
    // if P2 doesn't exist but there are other data for this satellite: use blanks
    else if (GNSS_data.status[i]) fprintf(fd,"                ");
    // if any data written to the line: finish it
    if (GNSS_data.status[i]) fprintf(fd,"%s",lin_end);
  } 


}



