#include "decl.h"

void write_header(char rinex_header[18][81],double GPS_secs,FILE *fd_rinex)

     // This function writes the header of the new rinex file.
     // See "RINEX: The Receiver Independent Exchange Format Version 2.10"
     // by Werner Gurtner, AIUB, for definitions.
     //
     // Input  rinex_header : rinex file header template
     //        GPS_secs     : time of first observation, GPS seconds
     //        fd_rinex     : rinex file

{
  int i,dat[6];
  time_t tim;
  struct tm *proc_tim;


  // The header is alredy "almost" finished, the two remaining elements
  // are the time of program run, and the time of first observation

  // derive the system time at program run
  time(&tim);
  proc_tim=gmtime(&tim);
  // write the system time on the second line of the header
  // add "0" if the time elements are just one digit, i.e. < 10
  sprintf(rinex_header[1]+40,"%4d ",proc_tim->tm_year+1900);
  if (proc_tim->tm_mon+1 < 10)    sprintf(rinex_header[1]+44,"0%1d",proc_tim->tm_mon+1);
  else    sprintf(rinex_header[1]+44,"%2d",proc_tim->tm_mon+1);
  if (proc_tim->tm_mday < 10)    sprintf(rinex_header[1]+46,"0%1d ",proc_tim->tm_mday);
  else    sprintf(rinex_header[1]+46,"%2d ",proc_tim->tm_mday);
  if (proc_tim->tm_hour < 10)    sprintf(rinex_header[1]+49,"0%1d:",proc_tim->tm_hour);
  else    sprintf(rinex_header[1]+49,"%2d:",proc_tim->tm_hour);
  if (proc_tim->tm_min < 10)    sprintf(rinex_header[1]+52,"0%1d:",proc_tim->tm_min);
  else    sprintf(rinex_header[1]+52,"%2d:",proc_tim->tm_min);
  if (proc_tim->tm_sec < 10)    sprintf(rinex_header[1]+55,"0%1d",proc_tim->tm_sec);
  else    sprintf(rinex_header[1]+55,"%2d",proc_tim->tm_sec);
  strncpy(rinex_header[1]+57,"UTC",3);

  // convert the GPS seconds of first observation to
  // date and time elements
  s2ymdhms(dat, (long)(GPS_secs+0.3));
  // write the observation time on the 17th line of the header
  sprintf(rinex_header[16],"%6d%6d%6d%6d%6d%13.7f",
          dat[0],dat[1],dat[2],dat[3],dat[4],(double)dat[5]);
  // overwrite the '\0' character
  strncpy(rinex_header[16]+43," ",1);

  // write all 18 lines of the header the the rinex file
  for (i =0; i < 18; i++) fprintf(fd_rinex,"%s%s",rinex_header[i],lin_end);


}

